defmodule AdvertisementCreationContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table                                                                                
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)                            
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  given_ ~r/^I want to insert the following data$/, fn state, %{table_data: table} ->
    data = List.first(table)
    {
      :ok,
      state
      |> Map.put(:data, data)
    }
  end

  and_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])
    {:ok, state}
  end

  and_ ~r/^I want to create a new property advertisement$/, fn state ->
    navigate_to("/")
    click({:id, "add_advertisement"})
    {:ok, state}
  end

  and_ ~r/^I fill in the property advertisement form with valid data$/, fn state ->
    fill_field({:id, "title"}, state[:data][:title])
    fill_field({:id, "description"}, state[:data][:description])
    select_dropdown("type", state[:data][:type])
    select_dropdown("property_type", state[:data][:property_type])
    fill_field({:id, "location"}, state[:data][:location])
    fill_field({:id, "area"}, state[:data][:area])
    fill_field({:id, "room_count"}, state[:data][:room_count])
    fill_field({:id, "floor"}, state[:data][:floor])
    fill_field({:id, "floor_count"}, state[:data][:floor_count])
    fill_field({:id, "price"}, state[:data][:price])
    {:ok, state}
  end

  when_ ~r/^I click submit$/, fn state ->
    click({:id, "submit_button"})
    {:ok, state}
  end

  then_ ~r/^I should see a success message$/, fn state ->
    assert visible_in_page? ~r/Property created successfully!/
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the property advertisement page$/, fn state ->
    paths = String.split(current_path(), "/")
    # https://stackoverflow.com/a/6640851
    regex = ~r/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/
    assert Enum.at(paths, -1) =~ regex
    {:ok, state}
  end

  and_ ~r/^I fill in the property advertisement form with invalid data$/, fn state ->
    fill_field({:id, "title"}, "wrong")
    fill_field({:id, "description"}, "invalid_obvs")

    # Note: These can't be set invalid from the UI
    select_dropdown("type", state[:data][:type])
    select_dropdown("property_type", state[:data][:property_type])

    fill_field({:id, "location"}, "Tartu, Estonia")
    fill_field({:id, "area"}, "-10")
    fill_field({:id, "room_count"}, "1")
    fill_field({:id, "floor"}, "1")
    fill_field({:id, "floor_count"}, "0")
    fill_field({:id, "price"}, "-100")
    {:ok, state}
  end

  then_ ~r/^I should see error messages$/, fn state ->
    assert visible_in_page? ~r/must be greater than 0/
    assert visible_in_page? ~r/Floor count must be greater than floor/
    assert visible_in_page? ~r/must be greater than 0/
    {:ok, state}
  end

  and_ ~r/^I should still see the new property advertisement form$/, fn state ->
    assert visible_in_page? ~r/Add a new property listing/
    {:ok, state}
  end

  given_ ~r/^I am not logged in$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  and_ ~r/^I would like to create a new property advertisement$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  then_ ~r/^I should not see a link to create a new property advertisement$/, fn state ->
    assert not visible_in_page? ~r/Add a new property listing/
    {:ok, state}
  end

  and_ ~r/^I should be redirected back to home page when I try to access the link$/, fn state ->
    assert current_path() == "/"
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

  # https://stackoverflow.com/a/49861811
  defp select_dropdown(drop_down_id, option) do
    find_element(:css, "##{drop_down_id} option[value='#{option}']") |> click()
  end
end
