defmodule PropTrackrWeb.PropertiesControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Repo

  @valid_data %{
    title: "Apartment",
    description: "Small apartment",
    type: "rent",
    property_type: "apartment",
    location: "Tartu, Estonia",
    room_count: 1,
    area: 13.0,
    floor: 2,
    floor_count: 4,
    price: 430.0,
  }

  @invalid_data %{
    title: "wrong",
    description: "invalid_obvs",
    type: "rent",
    property_type: "apartment",
    location: "Tartu, Estonia",
    room_count: 0,
    area: -1.0,
    floor: -1,
    floor_count: -1,
    price: -1.0,
  }

  setup do
    user = %User{
      name: "Test",
      surname: "User",
      birth_date: "2000-01-01",
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: "testing",
      confirm_password: "testing",
    }
    user = Repo.insert!(user)

    {:ok, %{user: user}}
  end

  test "Authenticated user should be able to create a new property advertisement with valid data", %{ conn: conn, user: user } do
    conn = conn |> setup_session(user)
    conn = conn |> post("/properties", property: @valid_data)

    uuidv4_regex =  ~r/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/
    path = String.split(redirected_to(conn), "/")
    assert Enum.at(path, -1) =~ uuidv4_regex
    assert Enum.at(path, -2) == "properties"

    advertisement_reference = Enum.at(path, -1)

    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200) =~ "Property created successfully!"

    advertisement = Repo.get_by!(Property, reference: advertisement_reference)
    assert advertisement.title == @valid_data[:title]
    assert advertisement.description == @valid_data[:description]
    assert advertisement.type == String.to_atom(@valid_data[:type])
    assert advertisement.property_type == String.to_atom(@valid_data[:property_type])
    assert advertisement.location == @valid_data[:location]
    assert advertisement.room_count == @valid_data[:room_count]
    assert advertisement.area == @valid_data[:area]
    assert advertisement.floor == @valid_data[:floor]
    assert advertisement.floor_count == @valid_data[:floor_count]
    assert advertisement.price == @valid_data[:price]
  end

  test "Authenticated user should not be able to create an advertisement with invalid data",  %{ conn: conn, user: user } do
    conn = conn |> setup_session(user)
    conn = conn |> post("/properties", property: @invalid_data)

    assert html_response(conn, 200) =~ "must be greater than 0"
  end

  test "Authenticated user should not be able to create an advertisement with invalid floor values", %{ conn: conn, user: user } do
    conn = conn |> setup_session(user)

    data = %{ @invalid_data | floor: 2, floor_count: 1 }
    conn = conn |> post("/properties", property: data)

    assert html_response(conn, 200) =~ "Floor count must be greater than floor"
  end

  test "Authenticated user should not be able to create an advertisement with invalid floor count data on a house listing", %{ conn: conn, user: user } do
    conn = conn |> setup_session(user)

    data = %{ @invalid_data | property_type: :house, floor: 2, floor_count: 1 }
    conn = conn |> post("/properties", property: data)

    # Just assert that a redirect happened, don't care about data since it does not matter here
    assert redirected_to(conn) != "/properties"
  end

  test "Unauthenticated user should be redirected to the homepage", %{ conn: conn, user: user } do
    conn = post(conn, "/properties", property: @valid_data)
    assert redirected_to(conn) == "/"
  end

  test "Unauthenticated user should see a message when there are no advertisements", %{ conn: conn } do
    conn = get conn, "/"

    assert html_response(conn, 200) =~ "No advertisements at the moment"
  end

  test "Unauthenticated user should see a list of all properties", %{ conn: conn, user: user } do
    conn = conn |> setup_session(user)
    conn = conn |> post("/properties", property: @valid_data)

    conn = get conn, "/"
    assert html_response(conn, 200) =~ @valid_data[:title]
    assert html_response(conn, 200) =~ @valid_data[:description]
    assert html_response(conn, 200) =~ @valid_data[:location]
    assert html_response(conn, 200) =~ Float.to_string(@valid_data[:price])
    assert html_response(conn, 200) =~ Integer.to_string(@valid_data[:room_count])
    assert html_response(conn, 200) =~ Float.to_string(@valid_data[:area])
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
